@@ -59,10 +59,31 @@ gok_input_get_device_list (void)
 	  for (i = 0; i < num_devices; i++) {
 		  GokInput *pInput;
 		  if (devices[i].use == IsXExtensionDevice) {
+			  GSList *sl;
+			  GokInput *input;
+			  gboolean duplicate;
+
+			  duplicate = FALSE;
+			  for (sl = pList; sl; sl = sl->next) {
+
+			      input = (GokInput *)sl->data;
+			      if (!strcmp (devices[i].name, input->name)) {
+				duplicate = TRUE;
+				break;
+			      }
+			  }
+ 
 			  pInput = g_new0 (GokInput, 1);
 			  pInput->name = g_strdup (devices[i].name);
 			  pInput->info = &devices[i];
-			  pList = g_slist_prepend (pList, pInput);
+			  if (duplicate) {
+			     while (input->next)
+				input = input->next; 
+
+			     input->next = pInput;
+			  }
+                          else  
+			      pList = g_slist_prepend (pList, pInput);
 		  }
 	  }
   }
@@ -71,13 +92,15 @@ gok_input_get_device_list (void)
 }
 
 #ifdef HAVE_XINPUT
-static XDeviceInfo*
-find_device_info(Display	*display,
-		 char		*name,
-		 gboolean       extended_only)
+static GokInput*
+find_input (Display	*display,
+	    char	*name,
+	    gboolean     extended_only)
 {
 	int		i;
 	int		num_devices;
+	GokInput *input = NULL;
+	GokInput *temp;
 	
 	if (devices) {
 		XFreeDeviceList (devices);
@@ -91,10 +114,25 @@ find_device_info(Display	*display,
 			if (devices[i].use != IsXExtensionDevice)
 				continue;
 
-		if (!strcmp(devices[i].name, name))
-			return &devices[i];
+		if (!strcmp(devices[i].name, name)) {
+			temp = g_new0 (GokInput, 1);
+			temp->name = g_strdup (devices[i].name);
+			temp->info = &devices[i];
+
+			if (input == NULL)
+				input = temp;
+			else {
+				GokInput *temp1 = input;
+
+				while (temp1->next)
+					temp1 = temp1->next;	
+
+				temp1->next = temp;
+			}
+			
+		}
 	}
-	return NULL;
+	return input;
 }
 
 #endif
@@ -103,43 +141,48 @@ gboolean
 gok_input_init (GokInput *input, GdkFilterFunc filter_func)
 {
 #ifdef HAVE_XINPUT
-	XEventClass      event_list[20];
+	XEventClass      event_list[40];
 	int              i, number = 0; 
 	GtkWidget       *window = gok_main_get_main_window ();
 	Display         *display;
 	GdkWindow       *root;
+	GokInput        *temp;
+
+	temp = input;
+
+	while (temp) {
 	
-	if (!gok_input_open (input))
-	{
-		g_warning ("Cannot open input device!\n");
-		return FALSE;
-	}
-	
-	for (i=0; i < input->info->num_classes; i++) 
-	{
-		switch (input->device->classes[i].input_class) 
-		{
-		case KeyClass:
-			DeviceKeyPress(input->device, 
-				       gok_input_types[GOK_INPUT_TYPE_KEY_PRESS], 
-				       event_list[number]); number++;
-			DeviceKeyRelease(input->device, 
-					 gok_input_types[GOK_INPUT_TYPE_KEY_RELEASE], 
-					 event_list[number]); number++;
-			break;      
-		case ButtonClass:
-			DeviceButtonPress(input->device, 
-					  gok_input_types[GOK_INPUT_TYPE_BUTTON_PRESS], 
-					  event_list[number]); number++;
-			DeviceButtonRelease(input->device, 
-					    gok_input_types[GOK_INPUT_TYPE_BUTTON_RELEASE], 
-					    event_list[number]); number++;
-			break;
-		case ValuatorClass:
-			DeviceMotionNotify(input->device, 
-					   gok_input_types[GOK_INPUT_TYPE_MOTION], 
-					   event_list[number]); number++;
+		if (!gok_input_open (temp)) {
+			g_warning ("Cannot open input device!\n");
+			return FALSE;
 		}
+	
+		for (i = 0; i < temp->info->num_classes; i++) {
+			switch (temp->device->classes[i].input_class) 
+			{
+			case KeyClass:
+				DeviceKeyPress(temp->device, 
+					       gok_input_types[GOK_INPUT_TYPE_KEY_PRESS], 
+					       event_list[number]); number++;
+				DeviceKeyRelease(temp->device, 
+						 gok_input_types[GOK_INPUT_TYPE_KEY_RELEASE], 
+						 event_list[number]); number++;
+					 break;      
+			case ButtonClass:
+				DeviceButtonPress(temp->device, 
+						  gok_input_types[GOK_INPUT_TYPE_BUTTON_PRESS], 
+						  event_list[number]); number++;
+				DeviceButtonRelease(temp->device, 
+						    gok_input_types[GOK_INPUT_TYPE_BUTTON_RELEASE], 
+						    event_list[number]); number++;
+				break;
+			case ValuatorClass:
+				DeviceMotionNotify(temp->device, 
+						   gok_input_types[GOK_INPUT_TYPE_MOTION], 
+						   event_list[number]); number++;
+			}
+		}
+		temp = temp->next;
 	}
 	printf ("%d event types available\n", number);
 	
@@ -187,6 +230,10 @@ gok_input_get_extension_device_name (void)
 void
 gok_input_free (GokInput *pInput)
 {
+#ifdef HAVE_XINPUT
+	if (pInput->next)
+		gok_input_free (pInput->next);
+#endif
 	g_free (pInput->name);
 	g_free (pInput);
 }
@@ -197,17 +244,10 @@ gok_input_find_by_name (char *name, gboolean extended_only)
   GtkWidget *window = gok_main_get_main_window ();
   GokInput *input = NULL;
 #ifdef HAVE_XINPUT
-  XDeviceInfo *device = NULL;
   
   if (window && window->window) 
     {
-      device = find_device_info (GDK_WINDOW_XDISPLAY (window->window), name, extended_only);
-    }
-  if (device)
-    {
-      input = g_new0 (GokInput, 1); 
-      input->info = device;
-      input->name = g_strdup (device->name);
+      input = find_input (GDK_WINDOW_XDISPLAY (window->window), name, extended_only);
     }
 #else
   g_message ("Warning, XInput extension not present, cannot open input device.");
